const data = {
  name: 'Bangalore Traffic',
  children: [
    {
      name: 'Heavy Traffic',
      shape: 'circle',
      color: 'red',
      children: [
        {
          name: 'Silk Board Junction',
          shape: 'rect',
          color: 'orange',
        },
        {
          name: 'KR Puram Bridge',
          shape: 'rect',
          color: 'orange',
        },
      ],
    },
    {
      name: 'Moderate Traffic',
      shape: 'circle',
      color: 'yellow',
      children: [
        {
          name: 'Majestic',
          shape: 'rect',
          color: 'orange',
        },
        {
          name: 'Marathahalli',
          shape: 'rect',
          color: 'orange',
        },
      ],
    },
    {
      name: 'Light Traffic',
      shape: 'circle',
      color: 'green',
      children: [
        {
          name: 'Electronic City',
          shape: 'rect',
          color: 'orange',
        },
        {
          name: 'Manyata Tech Park',
          shape: 'rect',
          color: 'orange',
        },
      ],
    },
  ],
};

const additionalLinksHeavyTraffic = [
  { source: 'Silk Board Junction', target: 'Majestic' },
  { source: 'Silk Board Junction', target: 'Marathahalli' },
  { source: 'KR Puram Bridge', target: 'Majestic' },
  { source: 'KR Puram Bridge', target: 'Marathahalli' },
];

const margin = {
  top: 10,
  right: 250,
  bottom: 200,
  left: 200,
};
const width = 850 - margin.left - margin.right;
const height = 650 - margin.top - margin.bottom;

const tree = d3.tree().size([height, width]);

let nodes = d3.hierarchy(data, (node) => node.children);

nodes = tree(nodes);

const svgExists = d3.select('svg').empty();

if (svgExists) {
  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom)
    .style('background-color', 'lightgray'); // Set background color

  const g = svg
    .append('g') // append group
    .attr(
      'transform',
      'translate(' + margin.left + ',' + margin.top + ')',
    );

  const link = g
    .selectAll('.link')
    .data(nodes.descendants().slice(1))
    .enter()
    .append('path')
    .attr('class', 'link')
    .style('stroke', 'olive') // Set olive green stroke color
    .style('stroke-width', '2px') // Set stroke width
    .style('stroke-dasharray', '5,5') // Set dashed line style
    .attr('d', (d) => {
      return `
        M${d.y},${d.x}
        L${d.parent.y},${d.parent.x}
      `;
    });

  additionalLinksHeavyTraffic.forEach((link) => {
    const sourceNode = nodes.find(
      (node) => node.data.name === link.source,
    );
    const targetNode = nodes.find(
      (node) => node.data.name === link.target,
    );
    if (sourceNode && targetNode) {
      g.append('path')
        .attr('class', 'link additional-link')
        .style('stroke', 'olive') // Set olive green stroke color
        .style('stroke-width', '2px')
        .attr('d', () => {
          return `
            M${sourceNode.y},${sourceNode.x}
            L${targetNode.y},${targetNode.x}
          `;
        });
    }
  });

  const node = g
    .selectAll('.node')
    .data(nodes.descendants())
    .enter()
    .append('g')
    .attr(
      'class',
      (d) =>
        'node' +
        (d.children ? ' node--internal' : ' node--leaf'),
    )
    .attr(
      'transform',
      (d) => 'translate(' + d.y + ',' + d.x + ')',
    );

  node.each(function (d) {
    const shape = d.data.shape;
    if (shape === 'circle') {
      d3.select(this)
        .append('circle')
        .attr('r', 10) // radius of circle
        .style('stroke', (d) => d.data.color)
        .style('fill');
    }
  });

  node
    .append('text')
    .attr('dy', '.60em')
    .attr('x', (d) => (d.children ? -15 : 15))
    .attr('y', (d) =>
      d.children && d.depth !== 0 ? -15 : 0,
    )
    .style('text-anchor', (d) =>
      d.children ? 'end' : 'start',
    )
    .text((d) => d.data.name);
}
